@@ -197,7 +197,7 @@ public:
 
 void appendLong(mstring &s, long val);
 
-mstring CsBinToString(const uint8_t b[], unsigned short binLength);
+mstring BytesToHexString(const uint8_t b[], unsigned short binLength);
 bool CsAsciiToBin(const char *a, uint8_t b[], unsigned short binLength);
 
 typedef const unsigned char CUCHAR;
@@ -385,9 +385,23 @@ inline mstring unEscape(cmstring &s)
 {
 	mstring ret;
 	for(cmstring::const_iterator it=s.begin();it!=s.end();++it)
-		ret+=char(*it=='\\' ? unEscape(*it) : *it);
+	{
+		if(*it=='\\')
+		{
+			++it;
+			ret+=unEscape(*it);
+		}
+		else
+			ret+=*it;
+	}
 	return ret;
 }
 
+MYSTD::string BytesToHexString(const uint8_t sum[], unsigned short lengthBin);
+
+// STFU helpers, (void) casts are not effective for certain functions
+static inline void ignore_value (int i) { (void) i; }
+static inline void ignore_ptr (void* p) { (void) p; }
+
 #endif // _META_H
 
