@@ -107,6 +107,7 @@ MapNameToInt n2iTbl[] = {
 		,{ "KeepExtraVersions", &keepnver, NULL, 10}
 		,{ "UseWrap", &usewrap, NULL, 10}
 		,{ "FreshIndexMaxAge", &maxtempdelay, NULL, 10}
+		,{ "RedirMax", &redirmax, NULL, 10}
 
 		,{ "DirPerms", &dirperms, NULL, 8}
 		,{ "FilePerms", &fileperms, NULL, 8}
@@ -395,7 +396,7 @@ struct tHookHandler: public tRepoData::IHookHandler, public lockable
 		if (0 >= --m_nRefCnt)
 		{
 			//system(cmdRel.c_str());
-			downTimeNext= time(0) + downDuration;
+			downTimeNext= ::time(0) + downDuration;
 		}
 	}
 	virtual void JobConnect()
@@ -405,8 +406,8 @@ struct tHookHandler: public tRepoData::IHookHandler, public lockable
 		{
 			if(downTimeNext) // huh, already ticking? reset
 				downTimeNext=0;
-			else
-				system(cmdCon.c_str());
+			else if(system(cmdCon.c_str()))
+				aclog::err(tSS() << "Warning: " << cmdCon << " returned with error code.");
 		}
 	}
 };
@@ -838,8 +839,8 @@ void ReadBackendsFile(const string & sFile, const string &sRepName)
 		}
 		else
 		{
-			cerr << "Bad backend description, around line "
-					<< itor.reader.GetPositionDescription() << endl;
+			cerr << "Bad backend description, around line " << sFile << ":"
+					<< itor.reader.GetCurrentLine() << endl;
 			exit(2);
 		}
 	}
@@ -865,7 +866,7 @@ void ReadRewriteFile(const string & sFile, const string & sRepName)
 	filereader reader;
 	if(debug>4)
 		cout << "Reading rewrite file: " << sFile <<endl;
-	reader.OpenFile(sFile.c_str());
+	reader.OpenFile(sFile);
 	reader.CheckGoodState(true);
 	reader.AddEofLines();
 
@@ -889,8 +890,9 @@ void ReadRewriteFile(const string & sFile, const string & sRepName)
 			}
 			else
 			{
-				cout << "Parse error, invalid URL" << sLine << " on line "
-						<< reader.GetPositionDescription() <<endl;
+				cout << "Parse error, invalid URL " << sLine
+						<< " in line " << sFile << ":"<< reader.GetCurrentLine() << endl;
+
 				exit(2);
 			}
 			continue;
@@ -907,7 +909,7 @@ void ReadRewriteFile(const string & sFile, const string & sRepName)
 			if ( !paths.empty() && hosts.empty())
 			{
 				cout << "Parse error, missing Site: field around line "
-						<< reader.GetPositionDescription() <<endl;
+						<< sFile << ":"<< reader.GetCurrentLine() << endl;
 				exit(2);
 			}
 			for (tStrVecIterConst itHost=hosts.begin();
@@ -937,7 +939,9 @@ void ReadRewriteFile(const string & sFile, const string & sRepName)
 		}
 		else if(!ParseKeyValLine(sLine, key, val))
 		{
-			cerr << "Error parsing rewrite definitions, around line " << reader.GetPositionDescription() <<endl;
+			cerr << "Error parsing rewrite definitions, around line "
+					<< sFile << ":"<< reader.GetCurrentLine() << endl;
+
 			exit(1);
 		}
 		
@@ -1115,6 +1119,9 @@ void PostProcConfig()
 	   maxtempdelay=27;
    }
 
+   if(redirmax == RESERVED_DEFVAL)
+	   redirmax = forcemanaged ? 0 : 5;
+
    if (acfg::debug >= LOG_MORE)
 	{
 		for (UINT i = 0; i < _countof(n2sTbl); i++)
@@ -1200,7 +1207,8 @@ time_t ExecutePostponed()
 				if(acfg::debug & LOG_FLUSH)
 					aclog::flush();
 
-				system(hooks.cmdRel.c_str());
+				if(system(hooks.cmdRel.c_str()))
+					aclog::err(tSS() << "Warning: " << hooks.cmdRel << " returned with error code.");
 				hooks.downTimeNext = 0;
 			}
 			else // in future, use the soonest time
