@@ -17,6 +17,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <dirent.h>
+#include <regex.h>
 
 #include <map>
 #include <string>
@@ -1412,9 +1413,11 @@ tCacheMan::enumIndexType tCacheMan::GuessIndexTypeFromURL(const mstring &sPath)
 	stripSuffix(sPureIfileName, ".xz");
 	stripSuffix(sPureIfileName, ".lzma");
 	if (sPureIfileName=="Packages") // Debian's Packages file
-	return EIDX_PACKAGES;
-	if (endsWithSzAr(sPureIfileName, ".db.tar"))
+		return EIDX_PACKAGES;
+
+	if(endsWithSzAr(sPureIfileName, ".db") || endsWithSzAr(sPureIfileName, ".db.tar"))
 		return EIDX_ARCHLXDB;
+
 	if (sPureIfileName == "setup")
 		return EIDX_CYGSETUP;
 
@@ -1434,7 +1437,6 @@ tCacheMan::enumIndexType tCacheMan::GuessIndexTypeFromURL(const mstring &sPath)
 	if (sPureIfileName == "Index")
 		return endsWithSzAr(sPath, "i18n/Index") ? EIDX_TRANSIDX : EIDX_DIFFIDX;
 
-
 	return EIDX_UNSUPPORTED;
 }
 
