@@ -278,13 +278,8 @@ void con::WorkLoop() {
 
 				ldbg("Parsed REQUEST:" << m_pTmpHead->frontLine);
 				ldbg("Rest: " << inBuf.size());
-				job * j = new job(m_pTmpHead, this);
-				m_pTmpHead=NULL; // owned by job
-				j->PrepareDownload();
-				m_jobs2send.push_back(j);
-#ifdef DEBUG
-				m_nProcessedJobs++;
-#endif
+				AddNewJob(m_pTmpHead);
+				m_pTmpHead=NULL; // owned by job now
 			}
         	MYCATCH(bad_alloc&)
         	{
@@ -411,3 +406,15 @@ void con::__tlogstuff::reset(const MYSTD::string &pNewFile, const MYSTD::string
 	sumOut=0;
 }
 
+job* con::AddNewJob(header *pHead)
+				{
+					job * j = new job(pHead, this);
+
+					j->PrepareDownload();
+					m_jobs2send.push_back(j);
+#ifdef DEBUG
+					m_nProcessedJobs++;
+#endif
+					return j;
+
+}
