@@ -607,16 +607,9 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, int fd, tDljQueue &qActive)
 		else if (r == 0) // looks like a timeout
 		{
 			dbgline;
-
-			if(qActive.empty()) // there was nothing to do either
-			{
-				sErrorMsg = "500 Transfer timeout";
-				return EFLAG_HINT_SWITCH;
-			}
-
-			sErrorMsg = "500 Disconnected by peer";
-
-			return EFLAG_DISCON|EFLAG_JOB_BROKEN;
+			sErrorMsg = "500 Connection timeout";
+			// was there anything to do at all?
+			return qActive.empty() ? EFLAG_HINT_SWITCH : (EFLAG_DISCON|EFLAG_JOB_BROKEN);
 		}
 
 		if (FD_ISSET(m_wakepipe[0], &rfds))
