@@ -293,8 +293,9 @@ bool fileitem_base::SetupClean()
 	if(m_status>FIST_FRESH)
 		return false;
 	m_status=FIST_INITED;
-	::truncate(m_sPathAbs.c_str(), 0);
-	::truncate((m_sPathAbs+".head").c_str(), 0);
+	if(::truncate(m_sPathAbs.c_str(), 0) || ::truncate((m_sPathAbs+".head").c_str(), 0))
+		return false;
+
 	m_head.clear();
 	m_nSizeSeen=m_nSizeChecked=0;
 
