@@ -12,15 +12,10 @@
 #include "sha1.h"
 #include "csmapping.h"
 
-#define FLAG_PLAIN 1
-#define FLAG_GZ 2
-#define FLAG_BZ2 4
 #define FLAG_EOF 8
 #define FLAG_ERROR 16
 #define FLAG_COMPEOF 32
-#define FLAG_LZMA 64
 
-//#include <fstream>
 #include <iostream>
 
 #define BUFSIZE 20000
@@ -33,9 +28,151 @@
 
 using namespace MYSTD;
 
+
+class IDecompressor
+{
+public:
+	bool inited;
+	IDecompressor() : inited(false) {};
+	virtual ~IDecompressor() {};
+	virtual UINT UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
+	{ return FLAG_ERROR; }
+
+};
+
+#ifdef HAVE_LIBBZ2
+#include <bzlib.h>
+class tBzDec : public IDecompressor
+{
+	bz_stream strm;
+public:
+	tBzDec()
+	{
+		::memset(&strm, 0, sizeof(strm));
+		inited = (BZ_OK == BZ2_bzDecompressInit(&strm, 1, EXTREME_MEMORY_SAVING));
+	}
+	~tBzDec()
+	{
+		BZ2_bzDecompressEnd(&strm);
+		inited = false;
+	}
+	virtual UINT UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
+	{
+		strm.next_in=szInBuf+nBufPos;
+		strm.avail_in=nBufSize-nBufPos;
+		strm.next_out=UncompBuf.wptr();
+		strm.avail_out=UncompBuf.freecapa();
+
+	int ret=BZ2_bzDecompress(&strm);
+	if(ret==BZ_STREAM_END || ret==BZ_OK)
+	{
+		nBufPos += ( (nBufSize-nBufPos) - strm.avail_in);
+		unsigned int nGotBytes= UncompBuf.freecapa() - strm.avail_out;
+		UncompBuf.got(nGotBytes);
+		return ret == BZ_STREAM_END ? FLAG_COMPEOF : 0;
+	}
+	// or corrupted data?
+	return (FLAG_COMPEOF|FLAG_ERROR);
+	}
+};
+#else
+#define tBzDec IDecompressor
+#endif
+
+#ifdef HAVE_ZLIB
+#include <zlib.h>
+class tGzDec : public IDecompressor
+{
+	z_stream strm;
+public:
+	tGzDec()
+	{
+		::memset(&strm, 0, sizeof(strm));
+		inited = (Z_OK == inflateInit2(&strm, 47));
+	}
+	~tGzDec()
+	{
+		deflateEnd(&strm);
+		inited = false;
+	}
+	virtual UINT UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
+	{
+		strm.next_in = (uint8_t*) szInBuf + nBufPos;
+		strm.avail_in = nBufSize - nBufPos;
+		strm.next_out = (uint8_t*) UncompBuf.wptr();
+		strm.avail_out = UncompBuf.freecapa();
+
+		int ret = inflate(&strm, Z_NO_FLUSH);
+		if (ret == Z_STREAM_END || ret == Z_OK)
+		{
+			nBufPos += ((nBufSize - nBufPos) - strm.avail_in);
+			unsigned int nGotBytes = UncompBuf.freecapa() - strm.avail_out;
+			UncompBuf.got(nGotBytes);
+			return ret == Z_STREAM_END ? FLAG_COMPEOF : 0;
+		}
+		// or corrupted data?
+		return (FLAG_COMPEOF | FLAG_ERROR);
+	}
+};
+
+#else
+#define tGzDec IDecompressor
+#endif
+
+#ifdef HAVE_LZMA
+#include <lzma.h>
+
+class tXzDec : public IDecompressor
+{
+	lzma_stream strm;
+public:
+	tXzDec(bool lzmaFormat=false)
+	{
+		::memset(&strm, 0, sizeof(strm));
+		if(lzmaFormat)
+			inited = (LZMA_OK == lzma_alone_decoder(&strm,
+					EXTREME_MEMORY_SAVING ? 32000000 : MAX_VAL(uint64_t)));
+		else
+			inited = (LZMA_OK == lzma_stream_decoder(&strm,
+				EXTREME_MEMORY_SAVING ? 32000000 : MAX_VAL(uint64_t),
+						LZMA_TELL_UNSUPPORTED_CHECK | LZMA_CONCATENATED));
+	}
+	~tXzDec()
+	{
+		lzma_end(&strm);
+		inited = false;
+	}
+	virtual UINT UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
+	{
+		strm.next_in = (uint8_t*) szInBuf + nBufPos;
+		strm.avail_in = nBufSize - nBufPos;
+		strm.next_out = (uint8_t*) UncompBuf.wptr();
+		strm.avail_out = UncompBuf.freecapa();
+
+		lzma_ret ret=lzma_code(&strm, LZMA_RUN);
+		if (ret == LZMA_STREAM_END || ret == LZMA_OK)
+		{
+			nBufPos += ((nBufSize - nBufPos) - strm.avail_in);
+			unsigned int nGotBytes = UncompBuf.freecapa() - strm.avail_out;
+			UncompBuf.got(nGotBytes);
+			return ret == LZMA_STREAM_END ? FLAG_COMPEOF : 0;
+		}
+		// or corrupted data?
+		return (FLAG_COMPEOF | FLAG_ERROR);
+	}
+};
+#else
+class tXzDec : public IDecompressor
+{
+public:
+	tXzDec() {}
+	tXzDec(bool) {}
+};
+#endif
+
 filereader::filereader() 
 :
-	flags(0),
+	flags(FLAG_ERROR|FLAG_EOF),
 	m_szFileBuf((char*)MAP_FAILED),
 	m_nBufSize(0),
 	m_nBufPos(0),
@@ -44,98 +181,77 @@ filereader::filereader()
 	m_nEofLines(0)
 {
 };
-	
 
-string filereader::GetPureFilename()
-{
-	string x=m_sOrigName;
-	tStrPos p=x.rfind('/'); 
-	if(stmiss!=p)
-		x.erase(0, p+1);
-	if(flags&FLAG_GZ)
-		x.erase(x.length()-3);
-	else if(flags&FLAG_BZ2)
-		x.erase(x.length()-4);
-	return x;
-}
+static const uint8_t gzMagic[] =
+{ 0x1f, 0x8b, 0x8 }, bz2Magic[] =
+{ 'B', 'Z', 'h' }, xzMagic[] =
+{ 0xfd, '7', 'z', 'X', 'Z', 0x0 },
+lzmaMagic[] = {0x5d, 0, 0, 0x80};
 
 bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 {
 	Close(); // reset to clean state
 	
-	m_sOrigName=sFilename;
-	m_fd=open(sFilename.c_str(), O_RDONLY);
-	
-	if(m_fd<0)
-		goto ofail;
+	m_fd = open(sFilename.c_str(), O_RDONLY);
 
-	if(bNoMagic)
-		flags|=FLAG_PLAIN;
-	else if(endsWithSzAr(sFilename, ".bz2"))
-	{
-#ifdef HAVE_LIBBZ2
-		m_bzStream.bzalloc = NULL;
-		m_bzStream.bzfree = NULL;
-		m_bzStream.opaque = NULL;
-		if(BZ_OK!=BZ2_bzDecompressInit ( & m_bzStream, 1, EXTREME_MEMORY_SAVING))
-			goto ofail;
-		
-		// ok, now can mark for processing and cleanup
-		flags |= FLAG_BZ2;
-		m_UncompBuf.init(BUFSIZE);
-#else
-		goto ofail;
-#endif
-	}
-	else if(endsWithSzAr(sFilename, ".gz"))
-	{
-#ifdef HAVE_ZLIB
-		memset(&m_zStream, 0, sizeof(m_zStream));
-				
-		if(Z_OK != inflateInit2(&m_zStream, 47))
-	    	goto ofail;
-		
-	    flags |= FLAG_GZ;
-	    m_UncompBuf.init(BUFSIZE);
-#else
-	    goto ofail;
-#endif
-	}
-	else if(endsWithSzAr(sFilename, ".lzma") || endsWithSzAr(sFilename, ".xz"))
-	{
-#ifdef HAVE_LZMA
-		memset(&m_xzStream, 0, sizeof(m_xzStream));
-		if(LZMA_OK != lzma_stream_decoder (&m_xzStream,
-				EXTREME_MEMORY_SAVING ? 32000000 : MAX_VAL(uint64_t),
-				LZMA_TELL_UNSUPPORTED_CHECK | LZMA_CONCATENATED))
-	    	goto ofail;
+	if (m_fd < 0)
+		return false;
 
-		flags |= FLAG_LZMA;
-		m_UncompBuf.init(BUFSIZE);
-#else
-	    goto ofail;
-#endif
+	if (bNoMagic)
+		m_Dec.reset();
+	else if (endsWithSzAr(sFilename, ".bz2"))
+		m_Dec.reset(new tBzDec);
+	else if (endsWithSzAr(sFilename, ".gz"))
+		m_Dec.reset(new tGzDec);
+	else if(endsWithSzAr(sFilename, ".xz"))
+		m_Dec.reset(new tXzDec);
+	else if (endsWithSzAr(sFilename, ".lzma"))
+		m_Dec.reset(new tXzDec(true));
+	else // unknown... ok, probe it
+	{
+		filereader fh;
+		if (fh.OpenFile(sFilename, true))
+		{
+			if (fh.GetSize() < 10)
+				return false;
+			if (0 == memcmp(gzMagic, fh.GetBuffer(), _countof(gzMagic)))
+				m_Dec.reset(new tGzDec);
+			else if (0 == memcmp(bz2Magic, fh.GetBuffer(), _countof(bz2Magic)))
+				m_Dec.reset(new tBzDec);
+			else if (0 == memcmp(xzMagic, fh.GetBuffer(), _countof(xzMagic)))
+				m_Dec.reset(new tXzDec);
+			else if (0 == memcmp(lzmaMagic, fh.GetBuffer(), _countof(lzmaMagic)))
+				m_Dec.reset(new tXzDec(true));
+
+		}
 	}
-	else
-		flags |= FLAG_PLAIN;
-		
-	
+
+	if (m_Dec.get())
+	{
+		if(!m_Dec->inited)
+		{
+			//aclog::err("Unable to uncompress file, algo not available");
+			return false;
+		}
+		m_UncompBuf.init(BUFSIZE);
+	}
+
 	struct stat statbuf;
 	if(0!=fstat(m_fd, &statbuf))
-		goto ofail;
+		return false;
 
 	// LFS on 32bit? That's not good for mmap. Don't risk incorrect behaviour.
 	if(uint64_t(statbuf.st_size) >  MYSTD::numeric_limits<size_t>::max())
     {
         errno=EFBIG;
-		goto ofail; 
+        return false;
     }
 	
 	if(statbuf.st_size>0)
 	{
 		m_szFileBuf = (char*) mmap(0, statbuf.st_size, PROT_READ, MAP_SHARED, m_fd, 0);
 		if(m_szFileBuf==MAP_FAILED)
-			goto ofail;
+			return false;
 		m_nBufSize = statbuf.st_size;
 	}
 	else
@@ -151,21 +267,20 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 	
 	m_nBufPos=0;
 	m_nCurLine=0;
+	flags = 0;
 	return true;
-	
-	ofail:
-	
-	flags |= (FLAG_ERROR|FLAG_EOF);
-	return false;
 }
 
-bool filereader::CheckGoodState(bool bErrorsConsiderFatal) const
+bool filereader::CheckGoodState(bool bErrorsConsiderFatal, cmstring *reportFilePath) const
 {	
 	if (flags&FLAG_ERROR)
 	{
 		if(bErrorsConsiderFatal)
 		{
-			cerr << "Error opening file: "<< m_sOrigName << ", terminating."<<endl;
+			cerr << "Error opening file";
+			if(reportFilePath)
+				cerr << " " << *reportFilePath;
+			cerr << ", terminating."<<endl;
 			exit(EXIT_FAILURE);
 		}
 		return false;
@@ -177,38 +292,18 @@ void filereader::Close()
 {
 	m_nCurLine=0;
 	
-	if(m_szFileBuf!=MAP_FAILED)
+	if (m_szFileBuf != MAP_FAILED)
 	{
 		munmap(m_szFileBuf, m_nBufSize);
-		m_szFileBuf=(char*)MAP_FAILED;
+		m_szFileBuf = (char*) MAP_FAILED;
 	}
-			
-  /*
-	if(m_fd>=0)
-	{
-		fsync(m_fd);
-		forceclose(m_fd);
-	}
-  */
-  checkforceclose(m_fd);
-	
-#ifdef HAVE_LIBBZ2
-	if(flags&FLAG_BZ2)
-		BZ2_bzDecompressEnd (&m_bzStream);
-#endif
 
-#ifdef HAVE_ZLIB
-	if(flags&FLAG_GZ)
-		deflateEnd(&m_zStream);
-#endif
+	checkforceclose(m_fd);
+	m_Dec.reset();
 
-#ifdef HAVE_LZMA
-	if(flags&FLAG_LZMA)
-     lzma_end(&m_xzStream);
-#endif
-
-	flags=0;
 	m_nBufSize=0;
+
+	flags = (FLAG_ERROR|FLAG_EOF); // will be cleared in open method
 }
 
 filereader::~filereader() {
@@ -232,7 +327,7 @@ bool filereader::GetOneLine(string & sOut) {
 
 	maybe_got_more:
 	
-	if(flags&FLAG_PLAIN)
+	if(!m_Dec.get())
 	{
 		bCanRetry=false;
 		
@@ -292,7 +387,7 @@ bool filereader::GetOneLine(string & sOut) {
 		if(bCanRetry)
 		{
 			bCanRetry=false;
-			_UncompressMoreData();
+			UncompressMoreData();
 			goto maybe_got_more;
 		}
 		
@@ -302,7 +397,7 @@ bool filereader::GetOneLine(string & sOut) {
 	
 	sOut.assign(rbuf, nLineLen);
 	
-	if(flags&FLAG_PLAIN)
+	if(!m_Dec.get())
 		m_nBufPos+=nDropLen;
 	else
 		m_UncompBuf.drop(nDropLen);
@@ -312,7 +407,7 @@ bool filereader::GetOneLine(string & sOut) {
 }
 
 //! @return: new text buffer size
-inline void filereader::_UncompressMoreData() {
+inline void filereader::UncompressMoreData() {
 
 	// work with uncompressed buffer/window...
 	m_UncompBuf.move(); // get unused space if possible
@@ -328,83 +423,9 @@ inline void filereader::_UncompressMoreData() {
 		return;
 	}
 	
-#ifndef MINIBUILD
-	unsigned int nFeedLen=m_nBufSize-m_nBufPos;
-#endif
-	
-#ifdef HAVE_LIBBZ2
-	if(flags&FLAG_BZ2)
-	{
-		m_bzStream.next_in=m_szFileBuf+m_nBufPos;
-		m_bzStream.avail_in=nFeedLen;
-		m_bzStream.next_out=m_UncompBuf.wptr();
-		m_bzStream.avail_out=m_UncompBuf.freecapa();
+	UINT wtf=m_Dec->UncompMore(m_szFileBuf, m_nBufSize, m_nBufPos, m_UncompBuf);
+	flags |= wtf;
 
-		int ret=BZ2_bzDecompress(&m_bzStream);
-		if(ret==BZ_STREAM_END)
-		{
-			// remember this later
-			flags|=FLAG_COMPEOF;
-			ret=BZ_OK;
-		}
-		if(ret==BZ_OK)
-		{
-			m_nBufPos += (nFeedLen-m_bzStream.avail_in);
-			unsigned int nGotBytes= m_UncompBuf.freecapa() - m_bzStream.avail_out;
-			m_UncompBuf.got(nGotBytes);
-		}
-		// or corrupted data?
-		else flags|=(FLAG_COMPEOF|FLAG_ERROR);
-	}
-#endif
-#ifdef HAVE_ZLIB
-	if(flags&FLAG_GZ)
-	{
-		m_zStream.next_in=(Bytef*) m_szFileBuf+m_nBufPos;
-		m_zStream.avail_in=nFeedLen;
-		m_zStream.next_out=(Bytef*) m_UncompBuf.wptr();
-		m_zStream.avail_out=m_UncompBuf.freecapa();
-
-		int ret=inflate(&m_zStream, Z_NO_FLUSH);
-		if(ret==Z_STREAM_END)
-		{
-			flags|=FLAG_COMPEOF;
-			ret=Z_OK;
-		}
-		if(ret==Z_OK)
-		{ //ok, accept the data
-			m_nBufPos += (nFeedLen-m_zStream.avail_in);
-			unsigned int nGotBytes= m_UncompBuf.freecapa() - m_zStream.avail_out;
-			m_UncompBuf.got(nGotBytes);
-		}
-		// or corrupted data?
-		else flags|=(FLAG_COMPEOF|FLAG_ERROR);
-	}
-#endif
-#ifdef HAVE_LZMA
-	if(flags&FLAG_LZMA)
-	{
-		m_xzStream.next_in=(const uint8_t*)m_szFileBuf+m_nBufPos;
-		m_xzStream.avail_in=nFeedLen;
-		m_xzStream.next_out=(uint8_t*)m_UncompBuf.wptr();
-		m_xzStream.avail_out=m_UncompBuf.freecapa();
-
-		lzma_ret ret=lzma_code(&m_xzStream, LZMA_RUN);
-		if(ret==LZMA_STREAM_END)
-		{
-			flags|=FLAG_COMPEOF;
-			ret=LZMA_OK;
-		}
-		if(ret==LZMA_OK)
-		{ //ok, accept the data
-			m_nBufPos += (nFeedLen-m_xzStream.avail_in);
-			unsigned int nGotBytes= m_UncompBuf.freecapa() - m_xzStream.avail_out;
-			m_UncompBuf.got(nGotBytes);
-		}
-		// or corrupted data?
-		else flags|=(FLAG_COMPEOF|FLAG_ERROR);
-	}
-#endif
 }
 
 /*
@@ -508,7 +529,7 @@ bool filereader::GetChecksum(int csType, uint8_t out[], off_t &scannedSize, FILE
 	auto_ptr<csumBase> summer = csumBase::GetChecker(CSTYPES(csType));
 	scannedSize=0;
 	
-	if (flags&FLAG_PLAIN)
+	if(!m_Dec.get())
 	{
 		summer->add(m_szFileBuf, m_nBufSize);
 		//sha_update(&ctx, (SHA_BYTE*) m_szFileBuf, m_nBufSize);
@@ -520,7 +541,7 @@ bool filereader::GetChecksum(int csType, uint8_t out[], off_t &scannedSize, FILE
 	{
 		while (true)
 		{
-			_UncompressMoreData();
+			UncompressMoreData();
 			if (flags&FLAG_ERROR)
 				return false;
 			UINT nRest=m_UncompBuf.size();
@@ -558,7 +579,7 @@ void check_algos()
 	ap = csumBase::GetChecker(CSTYPE_MD5);
 	ap->add(testvec, sizeof(testvec) - 1);
 	ap->finish(out);
-	if (CsBinToString(out, 16) != "900150983cd24fb0d6963f7d28e17f72")
+	if (BytesToHexString(out, 16) != "900150983cd24fb0d6963f7d28e17f72")
 	{
 		cerr << "Incorrect MD5 implementation detected, check compilation settings!\n";
 		exit(EXIT_FAILURE);
@@ -611,7 +632,7 @@ bool CsAsciiToBin(const char *a, uint8_t b[], unsigned short binLength)
 	return true;
 }
 
-string CsBinToString(const uint8_t sum[], unsigned short lengthBin)
+string BytesToHexString(const uint8_t sum[], unsigned short lengthBin)
 {
 	string sRet;
 	char hexmap[] =
@@ -683,3 +704,17 @@ bool Bz2compressFile(const char *pathIn, const char*pathOut)
 }
 
 #endif
+
+/*
+ * Playing (z|bz|xz)cat
+int main(int argc, char **argv)
+{
+	filereader fh;
+	fh.OpenFile(argv[1]);
+	mstring sLine;
+	while(fh.GetOneLine(sLine))
+	{
+		cout << sLine <<endl;
+	}
+}
+*/
